import React, { useState, useEffect, useMemo, useCallback } from "react";
import { Grid, Checkbox, CircularProgress, FormLabel } from "@material-ui/core";
import MUIDataTable, { debounceSearchRender } from "mui-datatables";
import PageTitle from "../../../components/PageTitle";
import { useResourcesApi } from "../../../api";
import { defaultResourcesFilters } from "../../../constants/resourcesConstants";

const __ROWS_PER_PAGE_OPTIONS = [10, 20, 50, 100];

const ResourcesContainer = () => {
  const [state, setState] = useState({ loaded: false, loadedPages: null });
  const [filters, setFilters] = useState({ ...defaultResourcesFilters });
  const { getResources } = useResourcesApi();

  useEffect(() => {
    if (state.loadedPages && state.loadedPages.includes(filters.page)) return;

    getResources(filters, {
      onCompleted: (resources) => {
        const data = { ...resources, loaded: true };
        setState((prev) => ({
          ...prev,
          ...data,
          values: prev?.values ? [...prev.values, ...data.values] : data.values,
          loadedPages: prev?.loadedPages
            ? [...prev.loadedPages, data.page]
            : [data.page]
        }));
      }
    });
  }, [getResources, filters, state.loadedPages]);

  const columns = useMemo(
    () => [
      {
        label: "Code",
        name: "resourceCode",
        options: {
          display: true,
          draggable: true,
          download: false,
          print: false,
          searchable: true,
          sort: true,
          customFilterListOptions: {
            render: (v) => (v ? `Code contains: ${v}` : [])
          },
          filter: true,
          filterType: "textField",
          filterOptions: {
            logic: (_prop, _filterValue, _row) => false
          },
          filterList: undefined,
          hint: undefined
        }
      },
      {
        label: "Name",
        name: "resourceName",
        options: {
          display: true,
          draggable: true,
          download: false,
          print: false,
          searchable: true,
          sort: true,
          customFilterListOptions: {
            render: (v) => (v ? `Name contains: ${v}` : [])
          },
          filter: true,
          filterType: "textField",
          filterOptions: {
            logic: (_prop, _filterValue, _row) => false
          },
          filterList: undefined,
          hint: undefined
        }
      },
      {
        label: "Category",
        name: "categoryId",
        options: {
          display: true,
          draggable: true,
          download: false,
          print: false,
          searchable: true,
          sort: true,
          filter: true,
          filterType: "dropdown",
          filterOptions: {
            logic: (_prop, _filterValue, _row) => false
          },
          filterList: [],
          hint: undefined
        }
      },
      {
        label: "Secured",
        name: "secured",
        options: {
          display: true,
          draggable: true,
          download: false,
          print: false,
          searchable: true,
          sort: true,
          customBodyRender: (value, _tableMeta, _updateValue) => {
            return (
              <Checkbox
                disabled
                size="small"
                checked={value}
                style={{ padding: "0" }}
              />
            );
          },
          customFilterListOptions: {
            render: (v) => {
              const checked = v[0];
              return checked ? <span>Secured</span> : [];
            }
          },
          filter: true,
          filterType: "custom",
          filterOptions: {
            logic: (_prop, _filterValue, _row) => false,
            display: (filterList, onChange, index, column) => {
              return (
                <div>
                  <FormLabel>Secured</FormLabel>
                  <Checkbox
                    color="primary"
                    checked={filterList[index][0] || false}
                    onChange={(event) => {
                      filterList[index][0] = event.target.checked;
                      onChange(filterList[index], index, column);
                    }}
                  />
                </div>
              );
            }
          },
          filterList: undefined,
          hint: undefined
        }
      }
    ],
    []
  );

  const handleResetFilters = useCallback(() => {
    setFilters((prev) => ({ ...prev, ...defaultResourcesFilters }));
  }, [setFilters]);

  const handleFilterChange = useCallback(
    (changedColumn, filterList, type, changedColumnIndex, displayData) => {
      if (type === "reset") {
        handleResetFilters();
        return;
      }
      const filterValue = filterList[changedColumnIndex][0];
      setFilters((prev) => ({
        ...prev,
        [changedColumn]: filterValue
      }));
    },
    [handleResetFilters]
  );

  return (
    <>
      <PageTitle title="ResourcesX" />
      {state.loaded === false ? (
        <CircularProgress size={26} />
      ) : (
        <Grid container spacing={4}>
          <Grid item xs={12}>
            <MUIDataTable
              title="Resources"
              columns={columns}
              data={state.values ?? []}
              options={{
                filterType: "textField",
                expandableRows: false,
                print: false,
                selectableRows: "none",
                rowsPerPage: state.pageSize,
                rowsPerPageOptions: __ROWS_PER_PAGE_OPTIONS,
                count: state.totalCount,
                customSearchRender: debounceSearchRender(500),
                onChangePage: (currentPage) => {
                  setFilters((prev) => ({ ...prev, page: currentPage + 1 }));
                },
                onChangeRowsPerPage: (numberOfRows) => {
                  setFilters((prev) => ({ ...prev, pageSize: numberOfRows }));
                },
                onColumnSortChange: (changedColumn, direction) =>
                  setFilters((prev) => ({
                    ...prev,
                    sortBy: changedColumn,
                    sortDirection: direction
                  })),
                onFilterChange: handleFilterChange,
                onSearchChange: (text) =>
                  setFilters((prev) => ({ ...prev, fullTextSearch: text })),
                customSort: (data) => data,
                customSearch: () => true,
                setFilterChipProps: (_colIndex, _colName, _data) => {
                  return {
                    color: "primary",
                    variant: "outlined"
                  };
                }
              }}
            />
          </Grid>
        </Grid>
      )}
    </>
  );
};

export default ResourcesContainer;
